from .ButtonDTO import Button
from .ButtonAntecedentFunctions import ButtonAntecedentFunction
from ..DeviceEvaluationDTO import DeviceEvaluation
from datetime import datetime


class ButtonFunction(object):
    def __init__(self, redis):
        self.r = redis
        self.button_antecedent_functions = ButtonAntecedentFunction(redis)

    def register(self, user_id: str, device_id: str) -> str:
        try:
            result = "false"
            key_pattern = "device:" + device_id
            if self.r.exists(key_pattern + ":name") == 0:
                self.r.rpush("user:" + user_id + ":sensors", device_id)
                device = Button()
                device.id = device_id
                device_id_keys = self.r.lrange("user:" + user_id + ":sensors")
                device.name = "BUTTON " + str(len(device_id_keys))
                self.r.set(key_pattern + ":user_id", user_id)
                self.r.set(key_pattern + ":name", device.name)
                self.r.set(key_pattern + ":measure", device.measure)
                self.r.set(key_pattern + ":last_time_on", device.last_time_on)
                self.r.set(key_pattern + ":last_time_off", device.last_time_off)
                self.r.set(key_pattern + ":last_date_on", device.last_date_on)
                self.r.set(key_pattern + ":last_date_off", device.last_date_off)
                self.r.set(key_pattern + ":expiration", device.expiration)
                result = device
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def get_device(self, user_id, device_id):
        try:
            key_pattern = "device:" + device_id
            dto = Button()
            dto.id = device_id
            dto.name = self.r.get(key_pattern + ":name")
            dto.expiration = self.r.get(key_pattern + ":expiration")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules_id = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules_id:
                    rule_name = self.r.get("user:" + user_id + ":rule:" + rule_id + ":name")
                    dto.rules.append({"id": rule_id, "name": rule_name})
            if self.r.exists(key_pattern + ":measure") == 1:
                measure = self.r.get(key_pattern + ":measure")
                if measure == "-":
                    dto.measure = measure
                    dto.color = "yellow"
                    dto.status = "initialization"
                else:
                    dto.measure = measure
                    dto.color = "green"
                    dto.status = "connected"
            dto.last_time_on = self.r.get(key_pattern + ":last_time_on")
            dto.last_time_off = self.r.get(key_pattern + ":last_time_off")
            dto.last_date_on = self.r.get(key_pattern + ":last_date_on")
            dto.last_date_off = self.r.get(key_pattern + ":last_date_off")
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def update_device(self, new_device):
        try:
            dto = Button()
            dto.device_mapping(new_device)
            key_pattern = "device:" + dto.id
            self.r.set(key_pattern + ":name", dto.name)
            self.r.set(key_pattern + ":expiration", dto.expiration)
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_device(self, user_id, device_id):
        try:
            self.r.lrem("user:" + user_id + ":sensors", device_id)
            key_pattern = "device:" + device_id
            self.r.delete(key_pattern + ":name")
            self.r.delete(key_pattern + ":user_id")
            self.r.delete(key_pattern + ":measure")
            self.r.delete(key_pattern + ":last_time_on")
            self.r.delete(key_pattern + ":last_time_off")
            self.r.delete(key_pattern + ":last_date_on")
            self.r.delete(key_pattern + ":last_date_off")
            self.r.delete(key_pattern + ":expiration")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules:
                    self.button_antecedent_functions.delete_antecedent(user_id, rule_id, device_id)
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def measure_evaluation(self, device_id, measure):
        time_str = datetime.now().strftime("%H:%M:%S")
        date_str = datetime.now().strftime("%d/%m/%Y")
        key_pattern = "device:" + device_id
        expiration = int(self.r.get(key_pattern + ":expiration")) + 2
        self.r.setex(key_pattern + ":measure", expiration, measure)
        if measure == "on":
            self.r.set(key_pattern + ":last_time_on", time_str)
            self.r.set(key_pattern + ":last_date_on", date_str)
        else:
            self.r.set(key_pattern + ":last_time_off", time_str)
            self.r.set(key_pattern + ":last_date_off", date_str)
        return measure

    def device_evaluation(self, device_id, measure):
        output = DeviceEvaluation()
        key_pattern = "device:" + device_id
        if self.r.exists(key_pattern + ":user_id") == 1:
            output.user_id = self.r.get("device:" + device_id + ":user_id")
            output.measure = self.measure_evaluation(device_id, measure)
            output.device_id = device_id
            output.type = "antecedent"
            if self.r.exists(key_pattern + ":rules"):
                output.rules = self.r.lrange(key_pattern + ":rules")
        return output



from typing import List


class Clouds:
    all: int

    def __init__(self, percentage: int) -> None:
        self.all = percentage


class Coord:
    lon: float
    lat: float

    def __init__(self, lon: float, lat: float) -> None:
        self.lon = lon
        self.lat = lat


class Main:
    temp: float
    feels_like: float
    temp_min: float
    temp_max: float
    pressure: int
    humidity: int

    def __init__(self, temp: float, feels_like: float, temp_min: float, temp_max: float, pressure: int,
                 humidity: int) -> None:
        self.temp = temp
        self.feels_like = feels_like
        self.temp_min = temp_min
        self.temp_max = temp_max
        self.pressure = pressure
        self.humidity = humidity


class Sys:
    type: int
    id: int
    message: float
    country: str
    sunrise: int
    sunset: int

    def __init__(self, type_sys: int, id_sys: int, message: float, country: str, sunrise: int, sunset: int) -> None:
        self.type = type_sys
        self.id = id_sys
        self.message = message
        self.country = country
        self.sunrise = sunrise
        self.sunset = sunset


class Weather:
    id: int
    main: str
    description: str
    icon: str

    def __init__(self, id_weather: int, main: str, description: str, icon: str) -> None:
        self.id = id_weather
        self.main = main
        self.description = description
        self.icon = icon


class Wind:
    speed: float
    deg: int

    def __init__(self, speed: float, deg: int) -> None:
        self.speed = speed
        self.deg = deg


class WeatherResponse:
    coord: Coord
    weather: List[Weather]
    base: str
    main: Main
    visibility: int
    wind: Wind
    clouds: Clouds
    dt: int
    sys: Sys
    timezone: int
    id: int
    name: str
    cod: int

    def __init__(self, x) -> None:
        self.coord = x.coord
        self.weather = x.weather
        self.base = x.base
        self.main = x.main
        self.visibility = x.visibility
        self.wind = x.wind
        self.clouds = x.clouds
        self.dt = x.dt
        self.sys = x.sys
        self.timezone = x.timezone
        self.id = x.id_obj
        self.name = x.name
        self.cod = x.cod



from ..WeatherAntecedentDTO import TempAntecedent


class TempAntecedentFunction(object):
    def __init__(self, redis):
        self.r = redis

    def get_temp_antecedent(self, user_id, rule_id, device_id):
        dto = TempAntecedent()
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        dto.check_temp = self.r.get(key_pattern + ":check_temp")
        dto.temp_condition = self.r.get(key_pattern + ":temp_condition")
        dto.temp_start_value = self.r.get(key_pattern + ":temp_start_value")
        dto.temp_stop_value = self.r.get(key_pattern + ":temp_stop_value")
        dto.temp_evaluation = self.r.get(key_pattern + ":temp_evaluation")
        location_name = self.r.get("user:" + user_id + ":location:name")
        dto.temp_value = self.r.get("weather:" + location_name + ":temp")
        return dto

    def set_temp_antecedent(self, user_id, rule_id, device_id, dto: TempAntecedent):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.set(key_pattern + ":check_temp", dto.check_temp)
        self.r.set(key_pattern + ":temp_condition", dto.temp_condition)
        self.r.set(key_pattern + ":temp_start_value", dto.temp_start_value)
        self.r.set(key_pattern + ":temp_stop_value", dto.temp_stop_value)
        self.r.set(key_pattern + ":temp_evaluation", dto.temp_evaluation)

    def delete_temp_antecedent(self, user_id, rule_id, device_id):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.delete(key_pattern + ":check_temp")
        self.r.delete(key_pattern + ":temp_condition")
        self.r.delete(key_pattern + ":temp_start_value")
        self.r.delete(key_pattern + ":temp_stop_value")
        self.r.delete(key_pattern + ":temp_evaluation")

    def register_temp_antecedent(self, user_id, rule_id, device_id):
        dto = TempAntecedent()
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.set(key_pattern + ":check_temp", dto.check_temp)
        self.r.set(key_pattern + ":temp_condition", dto.temp_condition)
        self.r.set(key_pattern + ":temp_start_value", dto.temp_start_value)
        self.r.set(key_pattern + ":temp_stop_value", dto.temp_stop_value)
        self.r.set(key_pattern + ":temp_evaluation", dto.temp_evaluation)

    def temp_antecedent_evaluation(self, user_id, rule_id, device_id):
        try:
            dto = self.get_temp_antecedent(user_id, rule_id, device_id)
            evaluation = "true"
            if dto.check_temp == "true":
                evaluation = "false"
                if dto.temp_condition == "between":
                    if float(dto.temp_start_value) < float(dto.temp_value) <= float(dto.temp_stop_value):
                        evaluation = "true"
                elif dto.temp_condition == ">":
                    if float(dto.temp_value) > float(dto.temp_start_value):
                        evaluation = "true"
                elif dto.temp_condition == "<":
                    if float(dto.temp_value) < float(dto.temp_stop_value):
                        evaluation = "true"
                if dto.temp_evaluation != evaluation:
                    key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
                    self.r.set(key_pattern + ":temp_evaluation", evaluation)
            return evaluation
        except Exception as error:
            print(repr(error))
            return "false"



from .PhotocellDTO import Photocell
from .PhotocellAntecedentFunctions import PhotocellAntecedentFunction
from ..DeviceEvaluationDTO import DeviceEvaluation
from datetime import datetime


class PhotocellFunction(object):
    def __init__(self, redis):
        self.r = redis
        self.photocell_antecedent_functions = PhotocellAntecedentFunction(redis)

    def register(self, user_id, device_id):
        try:
            result = "false"
            key_pattern = "device:" + device_id
            if self.r.exists(key_pattern + ":name") == 0:
                self.r.rpush("user:" + user_id + ":sensors", device_id)
                device = Photocell()
                device.id = device_id
                device_id_keys = self.r.lrange("user:" + user_id + ":sensors")
                device.name = "PHOTOCELL " + str(len(device_id_keys))
                self.r.set(key_pattern + ":name", device.name)
                self.r.set(key_pattern + ":user_id", user_id)
                self.r.set(key_pattern + ":measure", device.measure)
                self.r.set(key_pattern + ":max_measure", device.max_measure)
                self.r.set(key_pattern + ":max_measure_time", device.max_measure_time)
                self.r.set(key_pattern + ":max_measure_date", device.max_measure_date)
                self.r.set(key_pattern + ":min_measure", device.min_measure)
                self.r.set(key_pattern + ":min_measure_time", device.min_measure_time)
                self.r.set(key_pattern + ":min_measure_date", device.min_measure_date)
                self.r.set(key_pattern + ":expiration", device.expiration)
                result = device
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def get_device(self, user_id, device_id):
        try:
            key_pattern = "device:" + device_id
            dto = Photocell()
            dto.id = device_id
            dto.name = self.r.get(key_pattern + ":name")
            dto.max_measure = self.r.get(key_pattern + ":max_measure")
            dto.max_measure_time = self.r.get(key_pattern + ":max_measure_time")
            dto.max_measure_date = self.r.get(key_pattern + ":max_measure_date")
            dto.min_measure = self.r.get(key_pattern + ":min_measure")
            dto.min_measure_time = self.r.get(key_pattern + ":min_measure_time")
            dto.min_measure_date = self.r.get(key_pattern + ":min_measure_date")
            dto.expiration = self.r.get(key_pattern + ":expiration")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules_id = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules_id:
                    rule_name = self.r.get("user:" + user_id + ":rule:" + rule_id + ":name")
                    dto.rules.append({"id": rule_id, "name": rule_name})
            if self.r.exists(key_pattern + ":measure") == 1:
                measure = self.r.get(key_pattern + ":measure")
                dto.measure = measure
                if measure == "-":
                    dto.color = "yellow"
                    dto.status = "initialization"
                else:
                    dto.color = "green"
                    dto.status = "connected"
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def update_device(self, new_device):
        try:
            dto = Photocell()
            dto.device_mapping(new_device)
            key_pattern = "device:" + dto.id
            self.r.set(key_pattern + ":name", dto.name)
            self.r.set(key_pattern + ":expiration", dto.expiration)
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_device(self, user_id, device_id):
        try:
            self.r.lrem("user:" + user_id + ":sensors", device_id)
            key_pattern = "device:" + device_id
            self.r.delete(key_pattern + ":name")
            self.r.delete(key_pattern + ":user_id")
            self.r.delete(key_pattern + ":measure")
            self.r.delete(key_pattern + ":max_measure")
            self.r.delete(key_pattern + ":max_measure_time")
            self.r.delete(key_pattern + ":max_measure_date")
            self.r.delete(key_pattern + ":min_measure")
            self.r.delete(key_pattern + ":min_measure_time")
            self.r.delete(key_pattern + ":min_measure_date")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules:
                    self.photocell_antecedent_functions.delete_antecedent(user_id, rule_id, device_id)
        except Exception as error:
            print(repr(error))
            return "error"

    def measure_evaluation(self, device_id, measure):
        key_pattern = "device:" + device_id
        expiration = int(self.r.get(key_pattern + ":expiration")) + 2
        self.r.setex(key_pattern + ":measure", expiration, measure)
        self.device_max_measure_range(device_id, measure)
        self.device_min_measure_range(device_id, measure)
        return measure

    def device_evaluation(self, device_id, measure):
        output = DeviceEvaluation()
        key_pattern = "device:" + device_id
        if self.r.exists(key_pattern + ":user_id") == 1:
            output.user_id = self.r.get("device:" + device_id + ":user_id")
            output.measure = self.measure_evaluation(device_id, measure)
            output.device_id = device_id
            output.type = "antecedent"
            if self.r.exists(key_pattern + ":rules"):
                output.rules = self.r.lrange(key_pattern + ":rules")
        return output

    def device_max_measure_range(self, device_id, measure):
        key_pattern = "device:" + device_id
        time_str = datetime.now().strftime("%H:%M")
        date_str = datetime.now().strftime("%d/%m/%Y")
        max_measure = self.r.get(key_pattern + ":max_measure")
        if max_measure != "-":
            if float(measure) > float(max_measure):
                self.r.set(key_pattern + ":max_measure", measure)
                self.r.set(key_pattern + ":max_measure_time", time_str)
                self.r.set(key_pattern + ":max_measure_date", date_str)
        else:
            self.r.set(key_pattern + ":max_measure", measure)
            self.r.set(key_pattern + ":max_measure_time", time_str)
            self.r.set(key_pattern + ":max_measure_date", date_str)

    def device_min_measure_range(self, device_id, measure):
        key_pattern = "device:" + device_id
        time_str = datetime.now().strftime("%H:%M")
        date_str = datetime.now().strftime("%d/%m/%Y")
        min_measure = self.r.get(key_pattern + ":min_measure")
        if min_measure != "-":
            if float(measure) < float(min_measure):
                self.r.set(key_pattern + ":min_measure", measure)
                self.r.set(key_pattern + ":min_measure_time", time_str)
                self.r.set(key_pattern + ":min_measure_date", date_str)
        else:
            self.r.set(key_pattern + ":min_measure", measure)
            self.r.set(key_pattern + ":min_measure_time", time_str)
            self.r.set(key_pattern + ":min_measure_date", date_str)



import requests
from .WeatherResponseDTO import WeatherResponse
from .LocationDTO import Location
from munch import DefaultMunch
from .WeatherDTO import Weather
from datetime import datetime
from ..DeviceId import WEATHER


class WeatherFunction(object):
    def __init__(self, redis, api_key, api_location_url, api_weather_url):
        self.r = redis
        self.api_key = api_key
        self.api_location_url = api_location_url
        self.api_weather_url = api_weather_url
        self.weather_refresh_cycle = 60 * 60

    def register(self, user_id):
        try:
            device_id = WEATHER + "-" + user_id
            key_pattern = "device:" + device_id
            if self.r.exists(key_pattern + ":name") == 0:
                self.r.set(key_pattern + ":name", "WEATHER")
                self.r.set(key_pattern + ":user_id", user_id)
                self.set_location(user_id, "Torino", "IT", "45.1333", "7.3667")
                return "true"
            else:
                return "false"
        except Exception as error:
            print(repr(error))
            return "error"

    def get_device(self, user_id, device_id):
        try:
            weather = Weather()
            weather.id = device_id
            weather.name = self.r.get("device:" + device_id + ":name")
            # location
            weather.location_name = self.r.get("user:" + user_id + ":location:name")
            location_key_pattern = "weather:location:" + weather.location_name
            weather.country = self.r.get(location_key_pattern + ":country")
            weather.lat = self.r.get(location_key_pattern + ":lat")
            weather.lon = self.r.get(location_key_pattern + ":lon")
            # weather
            key_pattern = "weather:" + weather.location_name
            weather.icon = list(self.r.smembers(key_pattern + ":icon"))
            weather.temp = self.r.get(key_pattern + ":temp")
            weather.temp_max = self.r.get(key_pattern + ":temp_max")
            weather.temp_min = self.r.get(key_pattern + ":temp_min")
            weather.pressure = self.r.get(key_pattern + ":pressure")
            weather.humidity = self.r.get(key_pattern + ":humidity")
            weather.clouds = self.r.get(key_pattern + ":clouds")
            weather.sunrise = self.r.get(key_pattern + ":sunrise")
            weather.sunset = self.r.get(key_pattern + ":sunset")
            weather.wind_speed = self.r.get(key_pattern + ":wind_speed")
            weather.last_time_update = self.r.get(key_pattern + ":last_time_update")
            weather.last_date_update = self.r.get(key_pattern + ":last_date_update")
            if self.r.exists("device:" + device_id + ":rules") == 1:
                rules_id = self.r.lrange("device:" + device_id + ":rules")
                for rule_id in rules_id:
                    rule_name = self.r.get("user:" + user_id + ":rule:" + rule_id + ":name")
                    weather.rules.append({"id": rule_id, "name": rule_name})
            return weather
        except Exception as error:
            print(repr(error))
            return "error"

    def update_device(self, new_device):
        try:
            dto = Weather()
            dto.device_mapping(new_device)
            key_pattern = "device:" + dto.id
            self.r.set(key_pattern + ":name", dto.name)
        except Exception as error:
            print(repr(error))
            return "error"

    def set_location(self, user_id, location_name, country, lat, lon):
        try:
            self.check_unused_locations(user_id)
            self.r.set("user:" + user_id + ":location:name", location_name)
            key_pattern = "weather:location:" + location_name
            self.r.set(key_pattern + ":country", country)
            self.r.set(key_pattern + ":lat", lat)
            self.r.set(key_pattern + ":lon", lon)
            self.r.sadd("weather:location:names", location_name)
            self.update_weather(location_name)
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def check_unused_locations(self, user_id):
        try:
            if self.r.exists("user:" + user_id + ":location:name") == 1:
                old_location_name = self.r.get("user:" + user_id + ":location:name")
                self.r.set("user:" + user_id + ":location:name", "")
                all_users_location = set(self.r.scan("user:*:location:name"))
                if old_location_name not in all_users_location:
                    self.delete_weather(old_location_name)
        except Exception as error:
            print(repr(error))

    def get_location(self, user_id):
        try:
            location_name = self.r.get("user:" + user_id + ":location:name")
            key_pattern = "weather:location:" + location_name
            country = self.r.get(key_pattern + ":country")
            lat = self.r.get(key_pattern + ":lat")
            lon = self.r.get(key_pattern + ":lon")
            output = Location(location_name, country, lat, lon)
            return output
        except Exception as error:
            print(repr(error))
            return "error"

    def search_new_location(self, location_name):
        try:
            r = requests.get(self.api_location_url, params={'q': location_name, 'limit': 5, 'appid': self.api_key})
            data = r.json()
            return data
        except Exception as error:
            print(repr(error))
            return "error"

    def update_weather(self, location_name):
        try:
            key_pattern = "weather:" + location_name
            if self.r.exists(key_pattern + ":name") == 1:
                last_time_update_str = self.r.get(key_pattern + ":last_time_update")
                last_date_update_str = self.r.get(key_pattern + ":last_date_update")
                datetime_str = last_date_update_str + " " + last_time_update_str
                date_time_obj = datetime.strptime(datetime_str, '%d/%m/%Y %H:%M')
                delta_time = datetime.now() - date_time_obj
                if delta_time.total_seconds() > self.weather_refresh_cycle:
                    self.adapter_api_weather(location_name)
            else:
                self.adapter_api_weather(location_name)
        except Exception as error:
            print(repr(error))
            return "error"

    def adapter_api_weather(self, location_name):
        try:
            print("WEATHER API GET")
            location_key_pattern = "weather:location:" + location_name
            lat = self.r.get(location_key_pattern + ":lat")
            lon = self.r.get(location_key_pattern + ":lon")
            r = requests.get(self.api_weather_url,
                             params={'units': 'metric', 'lat': lat, 'lon': lon, 'appid': self.api_key})
            data = r.json()
            dto = WeatherResponse(DefaultMunch.fromDict(data))
            dto.name = location_name
            self.save_weather(dto)
        except Exception as error:
            print(repr(error))
            return "error"

    def save_weather(self, dto: WeatherResponse):
        key_pattern = "weather:" + dto.name
        self.r.set(key_pattern + ":name", dto.name)
        self.r.delete(key_pattern + ":icon")
        for i in range(0, len(dto.weather)):
            icon = list(dto.weather[i].icon)
            icon[-1] = "d"
            icon_d = "".join(icon)
            self.r.sadd(key_pattern + ":icon", icon_d)
        self.r.set(key_pattern + ":temp", str(dto.main.temp))
        self.r.set(key_pattern + ":temp_max", str(dto.main.temp_max))
        self.r.set(key_pattern + ":temp_min", str(dto.main.temp_min))
        self.r.set(key_pattern + ":pressure", str(dto.main.pressure))
        self.r.set(key_pattern + ":humidity", str(dto.main.humidity))
        self.r.set(key_pattern + ":clouds", str(dto.clouds.all))
        self.r.set(key_pattern + ":sunrise", str(dto.sys.sunrise))
        self.r.set(key_pattern + ":sunset", str(dto.sys.sunset))
        self.r.set(key_pattern + ":wind_speed", str(dto.wind.speed))
        self.r.set(key_pattern + ":last_time_update", datetime.now().strftime("%H:%M"))
        self.r.set(key_pattern + ":last_date_update", datetime.now().strftime("%d/%m/%Y"))

    def delete_weather(self, location_name):
        self.r.srem("weather:location:names", location_name)
        location_key_pattern = "weather:location:" + location_name
        self.r.delete(location_key_pattern + ":lat")
        self.r.delete(location_key_pattern + ":lon")
        self.r.delete(location_key_pattern + ":country")
        key_pattern = "weather:" + location_name
        if self.r.exists(key_pattern + ":name") == 1:
            self.r.delete(key_pattern + ":name")
            self.r.delete(key_pattern + ":icon")
            self.r.delete(key_pattern + ":temp")
            self.r.delete(key_pattern + ":temp_max")
            self.r.delete(key_pattern + ":temp_min")
            self.r.delete(key_pattern + ":pressure")
            self.r.delete(key_pattern + ":humidity")
            self.r.delete(key_pattern + ":clouds")
            self.r.delete(key_pattern + ":sunrise")
            self.r.delete(key_pattern + ":sunset")
            self.r.delete(key_pattern + ":wind_speed")




class IconAntecedent(object):
    def __init__(self):
        self.icon_value = []
        self.check_icon = "false"
        self.icon_condition = "between"
        self.icon_start_value = []
        self.icon_evaluation = "false"


class TempAntecedent(object):
    def __init__(self):
        self.temp_value = ""
        self.temp_unit = "Celsius"
        self.check_temp = "false"
        self.temp_condition = "between"
        self.temp_start_value = "-10"
        self.temp_stop_value = "40"
        self.temp_evaluation = "false"


class PressureAntecedent(object):
    def __init__(self):
        self.pressure_value = ""
        self.pressure_unit = "hPa"
        self.check_pressure = "false"
        self.pressure_condition = "between"
        self.pressure_start_value = "960"
        self.pressure_stop_value = "1060"
        self.pressure_evaluation = "false"


class HumidityAntecedent(object):
    def __init__(self):
        self.humidity_value = ""
        self.humidity_unit = "%"
        self.check_humidity = "false"
        self.humidity_condition = "between"
        self.humidity_start_value = "0"
        self.humidity_stop_value = "100"
        self.humidity_evaluation = "false"


class CloudAntecedent(object):
    def __init__(self):
        self.clouds_value = ""
        self.clouds_unit = "%"
        self.check_clouds = "false"
        self.clouds_condition = "between"
        self.clouds_start_value = "0"
        self.clouds_stop_value = "100"
        self.cloud_evaluation = "false"


class TimeAntecedent(object):
    def __init__(self):
        self.time_value = ""
        self.sunrise = ""
        self.sunset = ""
        self.check_time = "false"
        self.time_condition = "between"
        self.time_start_value = "-"
        self.time_stop_value = "-"
        self.time_evaluation = "false"


class WindAntecedent(object):
    def __init__(self):
        self.wind_speed_value = ""
        self.wind_speed_unit = "meter/sec"
        self.check_wind_speed = "false"
        self.wind_speed_condition = "between"
        self.wind_speed_start_value = "0"
        self.wind_speed_stop_value = "30"
        self.wind_speed_evaluation = "false"


class WeatherAntecedent(object):
    def __init__(self):
        self.device_id = ""
        self.device_name = ""
        self.icon_value = []
        self.temp_value = ""
        self.temp_unit = "Celsius"
        self.pressure_value = ""
        self.pressure_unit = "hPa"
        self.humidity_value = ""
        self.humidity_unit = "%"
        self.clouds_value = ""
        self.clouds_unit = "%"
        self.sunrise = ""
        self.sunset = ""
        self.wind_speed_value = ""
        self.wind_speed_unit = "meter/sec"
        self.check_icon = "true"
        self.icon_condition = ""
        self.icon_start_value = []
        self.check_temp = "true"
        self.temp_condition = ""
        self.temp_start_value = ""
        self.temp_stop_value = ""
        self.check_pressure = "true"
        self.pressure_condition = ""
        self.pressure_start_value = ""
        self.pressure_stop_value = ""
        self.check_humidity = "true"
        self.humidity_condition = ""
        self.humidity_start_value = ""
        self.humidity_stop_value = ""
        self.check_clouds = "true"
        self.clouds_condition = ""
        self.clouds_start_value = ""
        self.clouds_stop_value = ""
        self.check_time = "true"
        self.time_condition = ""
        self.time_start_value = ""
        self.time_stop_value = ""
        self.time_value = ""
        self.check_wind_speed = "true"
        self.wind_speed_condition = ""
        self.wind_speed_start_value = ""
        self.wind_speed_stop_value = ""
        self.icon_evaluation = "false"
        self.temp_evaluation = "false"
        self.pressure_evaluation = "false"
        self.humidity_evaluation = "false"
        self.cloud_evaluation = "false"
        self.time_evaluation = "false"
        self.wind_speed_evaluation = "false"
        self.evaluation = "false"

    def set_icon_antecedent(self, dto: IconAntecedent):
        self.check_icon = dto.check_icon
        self.icon_condition = dto.icon_condition
        self.icon_start_value = dto.icon_start_value
        self.icon_evaluation = dto.icon_evaluation
        self.icon_value = dto.icon_value

    def get_icon_antecedent(self):
        dto = IconAntecedent()
        dto.check_icon = self.check_icon
        dto.icon_condition = self.icon_condition
        dto.icon_start_value = self.icon_start_value
        dto.icon_evaluation = self.icon_evaluation
        dto.icon_value = self.icon_value
        return dto

    def set_temp_antecedent(self, dto: TempAntecedent):
        self.check_temp = dto.check_temp
        self.temp_condition = dto.temp_condition
        self.temp_start_value = dto.temp_start_value
        self.temp_stop_value = dto.temp_stop_value
        self.temp_evaluation = dto.temp_evaluation
        self.temp_value = dto.temp_value

    def get_temp_antecedent(self):
        dto = TempAntecedent()
        dto.check_temp = self.check_temp
        dto.temp_condition = self.temp_condition
        dto.temp_start_value = self.temp_start_value
        dto.temp_stop_value = self.temp_stop_value
        dto.temp_evaluation = self.temp_evaluation
        dto.temp_value = self.temp_value
        return dto

    def set_pressure_antecedent(self, dto: PressureAntecedent):
        self.check_pressure = dto.check_pressure
        self.pressure_condition = dto.pressure_condition
        self.pressure_start_value = dto.pressure_start_value
        self.pressure_stop_value = dto.pressure_stop_value
        self.pressure_evaluation = dto.pressure_evaluation
        self.pressure_value = dto.pressure_value

    def get_pressure_antecedent(self):
        dto = PressureAntecedent()
        dto.pressure_condition = self.pressure_condition
        dto.pressure_start_value = self.pressure_start_value
        dto.pressure_stop_value = self.pressure_stop_value
        dto.pressure_evaluation = self.pressure_evaluation
        dto.pressure_value = self.pressure_value
        return dto

    def set_humidity_antecedent(self, dto: HumidityAntecedent):
        self.check_humidity = dto.check_humidity
        self.humidity_condition = dto.humidity_condition
        self.humidity_start_value = dto.humidity_start_value
        self.humidity_stop_value = dto.humidity_stop_value
        self.humidity_evaluation = dto.humidity_evaluation
        self.humidity_value = dto.humidity_value

    def get_humidity_antecedent(self):
        dto = HumidityAntecedent()
        dto.check_humidity = self.check_humidity
        dto.humidity_condition = self.humidity_condition
        dto.humidity_start_value = self.humidity_start_value
        dto.humidity_stop_value = self.humidity_stop_value
        dto.humidity_evaluation = self.humidity_evaluation
        dto.humidity_value = self.humidity_value
        return dto

    def set_cloud_antecedent(self, dto: CloudAntecedent):
        self.check_clouds = dto.check_clouds
        self.clouds_condition = dto.clouds_condition
        self.clouds_start_value = dto.clouds_start_value
        self.clouds_stop_value = dto.clouds_stop_value
        self.cloud_evaluation = dto.cloud_evaluation
        self.clouds_value = dto.clouds_value

    def get_cloud_antecedent(self):
        dto = CloudAntecedent()
        dto.check_clouds = self.check_clouds
        dto.clouds_condition = self.clouds_condition
        dto.clouds_start_value = self.clouds_start_value
        dto.clouds_stop_value = self.clouds_stop_value
        dto.cloud_evaluation = self.cloud_evaluation
        dto.clouds_value = self.clouds_value
        return dto

    def set_time_antecedent(self, dto: TimeAntecedent):
        self.check_time = dto.check_time
        self.time_condition = dto.time_condition
        self.time_start_value = dto.time_start_value
        self.time_stop_value = dto.time_start_value
        self.time_evaluation = dto.time_evaluation
        self.sunrise = dto.sunrise
        self.sunset = dto.sunset
        self.time_value = dto.time_value

    def get_time_antecedent(self):
        dto = TimeAntecedent()
        dto.check_time = self.check_time
        dto.time_condition = self.time_condition
        dto.time_start_value = self.time_start_value
        dto.time_start_value = self.time_start_value
        dto.temp_evaluation = self.temp_evaluation
        dto.sunset = self.sunset
        dto.sunrise = self.sunrise
        dto.time_value = self.time_value
        return dto

    def set_wind_antecedent(self, dto: WindAntecedent):
        self.check_wind_speed = dto.check_wind_speed
        self.wind_speed_condition = dto.wind_speed_condition
        self.wind_speed_start_value = dto.wind_speed_start_value
        self.wind_speed_stop_value = dto.wind_speed_start_value
        self.wind_speed_evaluation = dto.wind_speed_evaluation
        self.wind_speed_value = dto.wind_speed_value

    def get_wind_antecedent(self):
        dto = WindAntecedent()
        dto.check_wind_speed = self.check_wind_speed
        dto.wind_speed_condition = self.wind_speed_condition
        dto.wind_speed_start_value = self.wind_speed_start_value
        dto.wind_speed_start_value = self.wind_speed_start_value
        dto.wind_speed_evaluation = self.wind_speed_evaluation
        dto.wind_speed_value = self.wind_speed_value
        return dto

    def antecedent_mapping(self, antecedent):
        self.device_id = antecedent["device_id"]
        self.device_name = antecedent["device_name"]
        self.check_icon = antecedent["check_icon"]
        self.icon_condition = antecedent["icon_condition"]
        self.icon_start_value = antecedent["icon_start_value"]
        self.check_temp = antecedent["check_temp"]
        self.temp_condition = antecedent["temp_condition"]
        self.temp_start_value = antecedent["temp_start_value"]
        self.temp_stop_value = antecedent["temp_stop_value"]
        self.check_pressure = antecedent["check_pressure"]
        self.pressure_condition = antecedent["pressure_condition"]
        self.pressure_start_value = antecedent["pressure_start_value"]
        self.pressure_stop_value = antecedent["pressure_stop_value"]
        self.check_humidity = antecedent["check_humidity"]
        self.humidity_condition = antecedent["humidity_condition"]
        self.humidity_start_value = antecedent["humidity_start_value"]
        self.humidity_stop_value = antecedent["humidity_stop_value"]
        self.check_clouds = antecedent["check_clouds"]
        self.clouds_condition = antecedent["clouds_condition"]
        self.clouds_start_value = antecedent["clouds_start_value"]
        self.clouds_stop_value = antecedent["clouds_stop_value"]
        self.check_time = antecedent["check_time"]
        self.time_condition = antecedent["time_condition"]
        self.time_start_value = antecedent["time_start_value"]
        self.time_stop_value = antecedent["time_stop_value"]
        self.check_wind_speed = antecedent["check_wind_speed"]
        self.wind_speed_condition = antecedent["wind_speed_condition"]
        self.wind_speed_start_value = antecedent["wind_speed_start_value"]
        self.wind_speed_stop_value = antecedent["wind_speed_stop_value"]

